package com.faraphel.tasks_valider.ui.screen.task

import android.app.Activity
import android.util.Log
import android.widget.Toast
import androidx.compose.foundation.layout.*
import androidx.compose.material3.Button
import androidx.compose.material3.Text
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.NavController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import com.faraphel.tasks_valider.connectivity.task.TaskClient
import com.faraphel.tasks_valider.database.entities.PersonEntity


@Composable
fun TaskSessionController(
    activity: Activity,
    client: TaskClient,
    user: PersonEntity,
) {
    val controller = rememberNavController()

    NavHost(
        navController = controller,
        startDestination = "main"
    ) {
        composable("main") {
            TaskSessionScreen(controller, activity, client, user)
        }
        composable("quick_validation") {
            QuickValidationScreen(controller, activity, client, user)
        }
    }
}


/**
 * This screen represent a session
 * @param activity the android activity
 * @param client an HTTP client that can communicate with the server
 */
@Composable
fun TaskSessionScreen(
    controller: NavController,
    activity: Activity,
    client: TaskClient,
    user: PersonEntity,
) {
    val students = remember { mutableStateOf<List<PersonEntity>?>(null) }
    val selectedStudent = remember { mutableStateOf<PersonEntity?>(null) }

    // if the groups are not yet defined, refresh the list
    if (students.value == null)
        return LaunchedEffect(true) {
            Thread { refreshStudents(activity, client, students) }.start()
        }

    if (selectedStudent.value != null)
        return TaskStudentScreen(
            activity,
            client,
            user,
            selectedStudent.value!!
        )

    Column(
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        // title
        Text(
            text = "Session",
            fontSize = 32.sp
        )

        // separator
        Spacer(modifier = Modifier.height(24.dp))

        // if the groups have already been defined, display them
        for (student in students.value!!) {
            Button(
                modifier = Modifier.fillMaxWidth().padding(vertical = 2.dp, horizontal = 16.dp),
                onClick = { selectedStudent.value = student },
            ) {
                Text(text = student.fullName())
            }
        }

        // separator
        Spacer(modifier = Modifier.weight(1f))

        // buttons
        Row {
            // quick validation
            Button(onClick = { controller.navigate("quick_validation") }) {
                Text("Quick Validation")
            }
        }
    }
}


// TODO(Faraphel): template this function ?
fun refreshStudents(
    activity: Activity,
    client: TaskClient,
    students: MutableState<List<PersonEntity>?>
) {
    try {
        // try to get all the persons in that session
        students.value = client.personApi.getAll()
    } catch (exception: Exception) {
        // in case of error, show a message
        return activity.runOnUiThread {
            Log.e("students", "$exception")
            Toast.makeText(activity, "Could not retrieve students.\n\n$exception", Toast.LENGTH_LONG).show()
        }
    }
}
